#!/bin/bash

# This is a simple calculator for the exclude and dir statement of the rpi's hal_gpio
# Data is based on this article: http://wiki.linuxcnc.org/cgi-bin/wiki.pl?RPi3BPreemptRT
# (C) Nikolaus Klepp <dr.klepp@gmx.at> - published under GPLv2 (which you should be able to find)

PIN="$1"
POUT="$2"


if [ $# -ne 2 ]; then
	echo "Usage: $(basename $0) <list of input pins> <list of output pins>"
	echo "Valid pins: 2 .. 27. You may use any seperator you like between list elements."
	echo "Example: $(basename $0)  17:27:22   2:3:4:12:14:15:18"
	echo "Example: $(basename $0) \"17 27 22\" \"2 3 4 12 14 15 18\""
	PIN="17 27 22"
	POUT="2 3 4 12 14 15 18"
fi

PIN=$(echo "$PIN"|tr -c '[0-9]' ' ')
POUT=$(echo "$POUT"|tr -c '[0-9]' ' ')

ILLEGAL=$(echo $PIN $POUT |tr ' ' '\n' | sort -n | uniq -c | awk '{ if (($1>1) || ($2<2) || ($2>27)) print $2 }' | tr '\n' ' ')
if [ ! -z "$ILLEGAL" ]; then
	echo "Error: These pins appear more than once or are not allowed: $ILLEGAL"
	exit 1
fi


EXC=$(echo $PIN $POUT | tr ' ' '\n' | awk 'BEGIN{a=2^26-1} {if ($1>0) a-=2^($1-2); } END{printf "%1.0f", a}')
DIR=$(echo $POUT | tr ' ' '\n' | awk 'BEGIN{a=0} {if ($1>0) a+=2^($1-2); } END{printf "%1.0f", a}')

echo "# Add this line to your \*.hal:"
echo "loadrt hal_pi_gpio dir=$DIR exclude=$EXC"

